@@ -21,9 +21,9 @@ import (
 	"reflect"
 	"testing"
 
-	"github.com/apache/arrow/go/v16/arrow"
-	"github.com/apache/arrow/go/v16/arrow/array"
-	"github.com/apache/arrow/go/v16/arrow/memory"
+	"github.com/apache/arrow/go/v17/arrow"
+	"github.com/apache/arrow/go/v17/arrow/array"
+	"github.com/apache/arrow/go/v17/arrow/memory"
 	"github.com/stretchr/testify/assert"
 )
 
@@ -94,7 +94,7 @@ func TestRecord(t *testing.T) {
 	if _, err := rec.SetColumn(0, col2_1); err == nil {
 		t.Fatalf("expected an error")
 	}
-	newRec, err := rec.SetColumn(1, col2_1);
+	newRec, err := rec.SetColumn(1, col2_1)
 	if err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
@@ -372,6 +372,45 @@ func TestRecordReader(t *testing.T) {
 	}
 }
 
+func TestRecordBuilderRespectsFixedSizeArrayNullability(t *testing.T) {
+	mem := memory.NewCheckedAllocator(memory.NewGoAllocator())
+	defer mem.AssertSize(t, 0)
+
+	cases := []struct {
+		assertion string
+		fields    []arrow.Field
+	}{
+		{
+			"nullable",
+			[]arrow.Field{{Name: "data", Type: arrow.FixedSizeListOf(1, arrow.PrimitiveTypes.Int32)}},
+		},
+		{
+			"not nullable",
+			[]arrow.Field{{Name: "data", Type: arrow.FixedSizeListOfNonNullable(1, arrow.PrimitiveTypes.Int32)}},
+		},
+	}
+	for _, c := range cases {
+		t.Run(c.assertion, func(t *testing.T) {
+			schema := arrow.NewSchema(c.fields, nil)
+			b := array.NewRecordBuilder(mem, schema)
+			defer b.Release()
+
+			lb := b.Field(0).(*array.FixedSizeListBuilder)
+			lb.Append(true)
+
+			vb := lb.ValueBuilder().(*array.Int32Builder)
+			vb.Append(10)
+
+			rec := b.NewRecord()
+			defer rec.Release()
+
+			if got, want := rec.Column(0).String(), "[[10]]"; got != want {
+				t.Fatalf("invalid record: got=%q, want=%q", got, want)
+			}
+		})
+	}
+}
+
 func TestRecordBuilder(t *testing.T) {
 	mem := memory.NewCheckedAllocator(memory.NewGoAllocator())
 	defer mem.AssertSize(t, 0)
