@@ -22,12 +22,12 @@ import (
 	"fmt"
 	"io"
 
-	"github.com/apache/arrow/go/v16/arrow"
-	"github.com/apache/arrow/go/v16/arrow/array"
-	"github.com/apache/arrow/go/v16/arrow/flight"
-	pb "github.com/apache/arrow/go/v16/arrow/flight/gen/flight"
-	"github.com/apache/arrow/go/v16/arrow/ipc"
-	"github.com/apache/arrow/go/v16/arrow/memory"
+	"github.com/apache/arrow/go/v17/arrow"
+	"github.com/apache/arrow/go/v17/arrow/array"
+	"github.com/apache/arrow/go/v17/arrow/flight"
+	pb "github.com/apache/arrow/go/v17/arrow/flight/gen/flight"
+	"github.com/apache/arrow/go/v17/arrow/ipc"
+	"github.com/apache/arrow/go/v17/arrow/memory"
 	"google.golang.org/grpc"
 	"google.golang.org/protobuf/proto"
 	"google.golang.org/protobuf/types/known/anypb"
@@ -243,6 +243,85 @@ func (c *Client) ExecuteSubstraitUpdate(ctx context.Context, plan SubstraitPlan,
 	return updateResult.GetRecordCount(), nil
 }
 
+// ExecuteIngest is for executing a bulk ingestion and only returns the number of affected rows.
+// The provided RecordReader will be retained for the duration of the call, but it is the caller's
+// responsibility to release the original reference.
+func (c *Client) ExecuteIngest(ctx context.Context, rdr array.RecordReader, reqOptions *ExecuteIngestOpts, opts ...grpc.CallOption) (int64, error) {
+	var (
+		err          error
+		desc         *flight.FlightDescriptor
+		stream       pb.FlightService_DoPutClient
+		wr           *flight.Writer
+		res          *pb.PutResult
+		updateResult pb.DoPutUpdateResult
+	)
+
+	cmd := (*pb.CommandStatementIngest)(reqOptions)
+
+	// Servers cannot infer defaults for these parameters, so we validate the request to ensure they are set.
+	if cmd.GetTableDefinitionOptions() == nil {
+		return 0, fmt.Errorf("cannot ExecuteIngest: invalid ExecuteIngestOpts, TableDefinitionOptions is required")
+	}
+	if cmd.GetTable() == "" {
+		return 0, fmt.Errorf("cannot ExecuteIngest: invalid ExecuteIngestOpts, Table is required")
+	}
+
+	if desc, err = descForCommand(cmd); err != nil {
+		return 0, err
+	}
+
+	if stream, err = c.Client.DoPut(ctx, opts...); err != nil {
+		return 0, err
+	}
+
+	wr = flight.NewRecordWriter(stream, ipc.WithAllocator(c.Alloc), ipc.WithSchema(rdr.Schema()))
+	defer wr.Close()
+
+	wr.SetFlightDescriptor(desc)
+
+	for rdr.Next() {
+		rec := rdr.Record()
+		err = wr.Write(rec)
+		if err == io.EOF {
+			// gRPC returns io.EOF if the error was generated by the server.
+			// The specific error will be retrieved in the server response.
+			// ref: https://pkg.go.dev/google.golang.org/grpc#ClientStream
+			break
+		}
+		if err != nil {
+			return 0, err
+		}
+	}
+
+	if err = rdr.Err(); err != nil {
+		return 0, err
+	}
+
+	if err = stream.CloseSend(); err != nil {
+		return 0, err
+	}
+
+	if res, err = stream.Recv(); err != nil {
+		return 0, err
+	}
+
+	if err = proto.Unmarshal(res.GetAppMetadata(), &updateResult); err != nil {
+		return 0, err
+	}
+
+	// Drain the stream. If ingestion was successful, no more messages should arrive.
+	// If there was a failure, the next message contains the error and the DoPutUpdateResult
+	// we recieved indicates a partial ingestion if the RecordCount is non-zero.
+	for {
+		_, err := stream.Recv()
+		if err == io.EOF {
+			return updateResult.GetRecordCount(), nil
+		} else if err != nil {
+			return updateResult.GetRecordCount(), err
+		}
+	}
+}
+
 // GetCatalogs requests the list of catalogs from the server and
 // returns a flightInfo object where the response can be retrieved
 func (c *Client) GetCatalogs(ctx context.Context, opts ...grpc.CallOption) (*flight.FlightInfo, error) {
@@ -1040,27 +1119,10 @@ func (p *PreparedStatement) Execute(ctx context.Context, opts ...grpc.CallOption
 		return nil, err
 	}
 
-	if p.hasBindParameters() {
-		pstream, err := p.client.Client.DoPut(ctx, opts...)
-		if err != nil {
-			return nil, err
-		}
-
-		wr, err := p.writeBindParameters(pstream, desc)
-		if err != nil {
-			return nil, err
-		}
-		if err = wr.Close(); err != nil {
-			return nil, err
-		}
-		pstream.CloseSend()
-
-		// wait for the server to ack the result
-		if _, err = pstream.Recv(); err != nil && err != io.EOF {
-			return nil, err
-		}
+	desc, err = p.bindParameters(ctx, desc, opts...)
+	if err != nil {
+		return nil, err
 	}
-
 	return p.client.getFlightInfo(ctx, desc, opts...)
 }
 
@@ -1080,25 +1142,9 @@ func (p *PreparedStatement) ExecutePut(ctx context.Context, opts ...grpc.CallOpt
 		return err
 	}
 
-	if p.hasBindParameters() {
-		pstream, err := p.client.Client.DoPut(ctx, opts...)
-		if err != nil {
-			return err
-		}
-
-		wr, err := p.writeBindParameters(pstream, desc)
-		if err != nil {
-			return err
-		}
-		if err = wr.Close(); err != nil {
-			return err
-		}
-		pstream.CloseSend()
-
-		// wait for the server to ack the result
-		if _, err = pstream.Recv(); err != nil && err != io.EOF {
-			return err
-		}
+	_, err = p.bindParameters(ctx, desc, opts...)
+	if err != nil {
+		return err
 	}
 
 	return nil
@@ -1126,25 +1172,9 @@ func (p *PreparedStatement) ExecutePoll(ctx context.Context, retryDescriptor *fl
 	}
 
 	if retryDescriptor == nil {
-		if p.hasBindParameters() {
-			pstream, err := p.client.Client.DoPut(ctx, opts...)
-			if err != nil {
-				return nil, err
-			}
-
-			wr, err := p.writeBindParameters(pstream, desc)
-			if err != nil {
-				return nil, err
-			}
-			if err = wr.Close(); err != nil {
-				return nil, err
-			}
-			pstream.CloseSend()
-
-			// wait for the server to ack the result
-			if _, err = pstream.Recv(); err != nil && err != io.EOF {
-				return nil, err
-			}
+		desc, err = p.bindParameters(ctx, desc, opts...)
+		if err != nil {
+			return nil, err
 		}
 	}
 	return p.client.Client.PollFlightInfo(ctx, desc, opts...)
@@ -1176,7 +1206,7 @@ func (p *PreparedStatement) ExecuteUpdate(ctx context.Context, opts ...grpc.Call
 		return
 	}
 	if p.hasBindParameters() {
-		wr, err = p.writeBindParameters(pstream, desc)
+		wr, err = p.writeBindParametersToStream(pstream, desc)
 		if err != nil {
 			return
 		}
@@ -1211,7 +1241,36 @@ func (p *PreparedStatement) hasBindParameters() bool {
 	return (p.paramBinding != nil && p.paramBinding.NumRows() > 0) || (p.streamBinding != nil)
 }
 
-func (p *PreparedStatement) writeBindParameters(pstream pb.FlightService_DoPutClient, desc *pb.FlightDescriptor) (*flight.Writer, error) {
+func (p *PreparedStatement) bindParameters(ctx context.Context, desc *pb.FlightDescriptor, opts ...grpc.CallOption) (*flight.FlightDescriptor, error) {
+	if p.hasBindParameters() {
+		pstream, err := p.client.Client.DoPut(ctx, opts...)
+		if err != nil {
+			return nil, err
+		}
+		wr, err := p.writeBindParametersToStream(pstream, desc)
+		if err != nil {
+			return nil, err
+		}
+		if err = wr.Close(); err != nil {
+			return nil, err
+		}
+		pstream.CloseSend()
+		if err = p.captureDoPutPreparedStatementHandle(pstream); err != nil {
+			return nil, err
+		}
+
+		cmd := pb.CommandPreparedStatementQuery{PreparedStatementHandle: p.handle}
+		desc, err = descForCommand(&cmd)
+		if err != nil {
+			return nil, err
+		}
+		return desc, nil
+	}
+	return desc, nil
+}
+
+// XXX: this does not capture the updated handle. Prefer bindParameters.
+func (p *PreparedStatement) writeBindParametersToStream(pstream pb.FlightService_DoPutClient, desc *pb.FlightDescriptor) (*flight.Writer, error) {
 	if p.paramBinding != nil {
 		wr := flight.NewRecordWriter(pstream, ipc.WithSchema(p.paramBinding.Schema()))
 		wr.SetFlightDescriptor(desc)
@@ -1234,6 +1293,29 @@ func (p *PreparedStatement) writeBindParameters(pstream pb.FlightService_DoPutCl
 	}
 }
 
+func (p *PreparedStatement) captureDoPutPreparedStatementHandle(pstream pb.FlightService_DoPutClient) error {
+	var (
+		result                  *pb.PutResult
+		preparedStatementResult pb.DoPutPreparedStatementResult
+		err                     error
+	)
+	if result, err = pstream.Recv(); err != nil && err != io.EOF {
+		return err
+	}
+	// skip if server does not provide a response (legacy server)
+	if result == nil {
+		return nil
+	}
+	if err = proto.Unmarshal(result.GetAppMetadata(), &preparedStatementResult); err != nil {
+		return err
+	}
+	handle := preparedStatementResult.GetPreparedStatementHandle()
+	if handle != nil {
+		p.handle = handle
+	}
+	return nil
+}
+
 // DatasetSchema may be nil if the server did not return it when creating the
 // Prepared Statement.
 func (p *PreparedStatement) DatasetSchema() *arrow.Schema { return p.datasetSchema }
