@@ -14,14 +14,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.arrow.vector;
 
 import static org.apache.arrow.vector.NullCheckingForGet.NULL_CHECKING_ENABLED;
 
 import java.math.BigDecimal;
 import java.nio.ByteOrder;
-
 import org.apache.arrow.memory.ArrowBuf;
 import org.apache.arrow.memory.BufferAllocator;
 import org.apache.arrow.memory.util.MemoryUtil;
@@ -38,11 +36,11 @@ import org.apache.arrow.vector.util.TransferPair;
 import org.apache.arrow.vector.validate.ValidateUtil;
 
 /**
- * DecimalVector implements a fixed width vector (16 bytes) of
- * decimal values which could be null. A validity buffer (bit vector) is
- * maintained to track which elements in the vector are null.
+ * DecimalVector implements a fixed width vector (16 bytes) of decimal values which could be null. A
+ * validity buffer (bit vector) is maintained to track which elements in the vector are null.
  */
-public final class DecimalVector extends BaseFixedWidthVector {
+public final class DecimalVector extends BaseFixedWidthVector
+    implements ValueIterableVector<BigDecimal> {
   public static final int MAX_PRECISION = 38;
   public static final byte TYPE_WIDTH = 16;
   private static final boolean LITTLE_ENDIAN = ByteOrder.nativeOrder() == ByteOrder.LITTLE_ENDIAN;
@@ -51,20 +49,20 @@ public final class DecimalVector extends BaseFixedWidthVector {
   private final int scale;
 
   /**
-   * Instantiate a DecimalVector. This doesn't allocate any memory for
-   * the data in vector.
+   * Instantiate a DecimalVector. This doesn't allocate any memory for the data in vector.
    *
    * @param name name of the vector
    * @param allocator allocator for memory management.
    */
-  public DecimalVector(String name, BufferAllocator allocator,
-                               int precision, int scale) {
-    this(name, FieldType.nullable(new ArrowType.Decimal(precision, scale, TYPE_WIDTH * 8)), allocator);
+  public DecimalVector(String name, BufferAllocator allocator, int precision, int scale) {
+    this(
+        name,
+        FieldType.nullable(new ArrowType.Decimal(precision, scale, TYPE_WIDTH * 8)),
+        allocator);
   }
 
   /**
-   * Instantiate a DecimalVector. This doesn't allocate any memory for
-   * the data in vector.
+   * Instantiate a DecimalVector. This doesn't allocate any memory for the data in vector.
    *
    * @param name name of the vector
    * @param fieldType type of Field materialized by this vector
@@ -75,8 +73,7 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Instantiate a DecimalVector. This doesn't allocate any memory for
-   * the data in vector.
+   * Instantiate a DecimalVector. This doesn't allocate any memory for the data in vector.
    *
    * @param field field materialized by this vector
    * @param allocator allocator for memory management.
@@ -94,8 +91,7 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Get minor type for this vector. The vector holds values belonging
-   * to a particular type.
+   * Get minor type for this vector. The vector holds values belonging to a particular type.
    *
    * @return {@link org.apache.arrow.vector.types.Types.MinorType}
    */
@@ -104,18 +100,16 @@ public final class DecimalVector extends BaseFixedWidthVector {
     return MinorType.DECIMAL;
   }
 
-
   /*----------------------------------------------------------------*
-   |                                                                |
-   |          vector value retrieval methods                        |
-   |                                                                |
-   *----------------------------------------------------------------*/
-
+  |                                                                |
+  |          vector value retrieval methods                        |
+  |                                                                |
+  *----------------------------------------------------------------*/
 
   /**
    * Get the element at the given index from the vector.
    *
-   * @param index   position of element
+   * @param index position of element
    * @return element at given index
    */
   public ArrowBuf get(int index) throws IllegalStateException {
@@ -126,11 +120,10 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Get the element at the given index from the vector and
-   * sets the state in holder. If element at given index
-   * is null, holder.isSet will be zero.
+   * Get the element at the given index from the vector and sets the state in holder. If element at
+   * given index is null, holder.isSet will be zero.
    *
-   * @param index   position of element
+   * @param index position of element
    */
   public void get(int index, NullableDecimalHolder holder) {
     if (isSet(index) == 0) {
@@ -147,7 +140,7 @@ public final class DecimalVector extends BaseFixedWidthVector {
   /**
    * Same as {@link #get(int)}.
    *
-   * @param index   position of element
+   * @param index position of element
    * @return element at given index
    */
   @Override
@@ -162,40 +155,34 @@ public final class DecimalVector extends BaseFixedWidthVector {
   /**
    * Same as {@link #getObject(int)} but does not check for null.
    *
-   * @param index   position of element
+   * @param index position of element
    * @return element at given index
    */
   public BigDecimal getObjectNotNull(int index) {
     return DecimalUtility.getBigDecimalFromArrowBuf(valueBuffer, index, scale, TYPE_WIDTH);
   }
 
-  /**
-   * Return precision for the decimal value.
-   */
+  /** Return precision for the decimal value. */
   public int getPrecision() {
     return precision;
   }
 
-  /**
-   * Return scale for the decimal value.
-   */
+  /** Return scale for the decimal value. */
   public int getScale() {
     return scale;
   }
 
-
   /*----------------------------------------------------------------*
-   |                                                                |
-   |          vector value setter methods                           |
-   |                                                                |
-   *----------------------------------------------------------------*/
-
+  |                                                                |
+  |          vector value setter methods                           |
+  |                                                                |
+  *----------------------------------------------------------------*/
 
   /**
    * Set the element at the given index to the given value.
    *
-   * @param index    position of element
-   * @param buffer   ArrowBuf containing decimal value.
+   * @param index position of element
+   * @param buffer ArrowBuf containing decimal value.
    */
   public void set(int index, ArrowBuf buffer) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -203,16 +190,15 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Set the decimal element at given index to the provided array of bytes.
-   * Decimal is now implemented as Native Endian. This API allows the user
-   * to pass a decimal value in the form of byte array in BE byte order.
+   * Set the decimal element at given index to the provided array of bytes. Decimal is now
+   * implemented as Native Endian. This API allows the user to pass a decimal value in the form of
+   * byte array in BE byte order.
    *
-   * <p>Consumers of Arrow code can use this API instead of first swapping
-   * the source bytes (doing a write and read) and then finally writing to
-   * ArrowBuf of decimal vector.
+   * <p>Consumers of Arrow code can use this API instead of first swapping the source bytes (doing a
+   * write and read) and then finally writing to ArrowBuf of decimal vector.
    *
-   * <p>This method takes care of adding the necessary padding if the length
-   * of byte array is less than 16 (length of decimal type).
+   * <p>This method takes care of adding the necessary padding if the length of byte array is less
+   * than 16 (length of decimal type).
    *
    * @param index position of element
    * @param value array of bytes containing decimal in big endian byte order.
@@ -249,11 +235,11 @@ public final class DecimalVector extends BaseFixedWidthVector {
       if (length <= TYPE_WIDTH) {
         // copy data from value to outAddress
         MemoryUtil.UNSAFE.copyMemory(
-                value,
-                MemoryUtil.BYTE_ARRAY_BASE_OFFSET,
-                null,
-                outAddress + DecimalVector.TYPE_WIDTH - length,
-                length);
+            value,
+            MemoryUtil.BYTE_ARRAY_BASE_OFFSET,
+            null,
+            outAddress + DecimalVector.TYPE_WIDTH - length,
+            length);
         // sign extend
         final byte pad = (byte) (value[0] < 0 ? 0xFF : 0x00);
         MemoryUtil.UNSAFE.setMemory(outAddress, DecimalVector.TYPE_WIDTH - length, pad);
@@ -267,9 +253,9 @@ public final class DecimalVector extends BaseFixedWidthVector {
   /**
    * Set the element at the given index to the given value.
    *
-   * @param index    position of element
-   * @param start    start index of data in the buffer
-   * @param buffer   ArrowBuf containing decimal value.
+   * @param index position of element
+   * @param start start index of data in the buffer
+   * @param buffer ArrowBuf containing decimal value.
    */
   public void set(int index, long start, ArrowBuf buffer) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -278,6 +264,7 @@ public final class DecimalVector extends BaseFixedWidthVector {
 
   /**
    * Sets the element at given index using the buffer whose size maybe <= 16 bytes.
+   *
    * @param index index to write the decimal to
    * @param start start of value in the buffer
    * @param buffer contains the decimal in native endian bytes
@@ -302,7 +289,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
         MemoryUtil.UNSAFE.setMemory(outAddress + length, DecimalVector.TYPE_WIDTH - length, pad);
       }
     } else {
-      MemoryUtil.UNSAFE.copyMemory(inAddress, outAddress + DecimalVector.TYPE_WIDTH - length, length);
+      MemoryUtil.UNSAFE.copyMemory(
+          inAddress, outAddress + DecimalVector.TYPE_WIDTH - length, length);
       // sign extend
       if (length < TYPE_WIDTH) {
         byte msb = MemoryUtil.UNSAFE.getByte(inAddress);
@@ -312,9 +300,9 @@ public final class DecimalVector extends BaseFixedWidthVector {
     }
   }
 
-
   /**
    * Sets the element at given index using the buffer whose size maybe <= 16 bytes.
+   *
    * @param index index to write the decimal to
    * @param start start of value in the buffer
    * @param buffer contains the decimal in big endian bytes
@@ -344,7 +332,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
         MemoryUtil.UNSAFE.setMemory(outAddress + length, DecimalVector.TYPE_WIDTH - length, pad);
       }
     } else {
-      MemoryUtil.UNSAFE.copyMemory(inAddress, outAddress + DecimalVector.TYPE_WIDTH - length, length);
+      MemoryUtil.UNSAFE.copyMemory(
+          inAddress, outAddress + DecimalVector.TYPE_WIDTH - length, length);
       // sign extend
       if (length < TYPE_WIDTH) {
         byte msb = MemoryUtil.UNSAFE.getByte(inAddress);
@@ -357,8 +346,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
   /**
    * Set the element at the given index to the given value.
    *
-   * @param index   position of element
-   * @param value   BigDecimal containing decimal value.
+   * @param index position of element
+   * @param value BigDecimal containing decimal value.
    */
   public void set(int index, BigDecimal value) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -369,8 +358,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
   /**
    * Set the element at the given index to the given value.
    *
-   * @param index   position of element
-   * @param value   long value.
+   * @param index position of element
+   * @param value long value.
    */
   public void set(int index, long value) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -378,12 +367,11 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Set the element at the given index to the value set in data holder.
-   * If the value in holder is not indicated as set, element in the
-   * at the given index will be null.
+   * Set the element at the given index to the value set in data holder. If the value in holder is
+   * not indicated as set, element in the at the given index will be null.
    *
-   * @param index   position of element
-   * @param holder  nullable data holder for value of element
+   * @param index position of element
+   * @param holder nullable data holder for value of element
    */
   public void set(int index, NullableDecimalHolder holder) throws IllegalArgumentException {
     if (holder.isSet < 0) {
@@ -399,8 +387,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
   /**
    * Set the element at the given index to the value set in data holder.
    *
-   * @param index   position of element
-   * @param holder  data holder for value of element
+   * @param index position of element
+   * @param holder data holder for value of element
    */
   public void set(int index, DecimalHolder holder) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -408,12 +396,11 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, ArrowBuf)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, ArrowBuf)} except that it handles the case when index is greater than
+   * or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param buffer  ArrowBuf containing decimal value.
+   * @param index position of element
+   * @param buffer ArrowBuf containing decimal value.
    */
   public void setSafe(int index, ArrowBuf buffer) {
     handleSafe(index);
@@ -421,9 +408,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #setBigEndian(int, byte[])} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #setBigEndian(int, byte[])} except that it handles the case when index is
+   * greater than or equal to existing value capacity {@link #getValueCapacity()}.
    */
   public void setBigEndianSafe(int index, byte[] value) {
     handleSafe(index);
@@ -431,13 +417,12 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, long, ArrowBuf)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, long, ArrowBuf)} except that it handles the case when index is greater
+   * than or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index    position of element
-   * @param start    start index of data in the buffer
-   * @param buffer   ArrowBuf containing decimal value.
+   * @param index position of element
+   * @param start start index of data in the buffer
+   * @param buffer ArrowBuf containing decimal value.
    */
   public void setSafe(int index, long start, ArrowBuf buffer) {
     handleSafe(index);
@@ -445,12 +430,11 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, BigDecimal)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, BigDecimal)} except that it handles the case when index is greater
+   * than or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param value   BigDecimal containing decimal value.
+   * @param index position of element
+   * @param value BigDecimal containing decimal value.
    */
   public void setSafe(int index, BigDecimal value) {
     handleSafe(index);
@@ -458,12 +442,11 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, long)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, long)} except that it handles the case when index is greater than or
+   * equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param value   long value.
+   * @param index position of element
+   * @param value long value.
    */
   public void setSafe(int index, long value) {
     handleSafe(index);
@@ -471,12 +454,11 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, NullableDecimalHolder)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, NullableDecimalHolder)} except that it handles the case when index is
+   * greater than or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param holder  nullable data holder for value of element
+   * @param index position of element
+   * @param holder nullable data holder for value of element
    */
   public void setSafe(int index, NullableDecimalHolder holder) throws IllegalArgumentException {
     handleSafe(index);
@@ -484,12 +466,11 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, DecimalHolder)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, DecimalHolder)} except that it handles the case when index is greater
+   * than or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param holder  data holder for value of element
+   * @param index position of element
+   * @param holder data holder for value of element
    */
   public void setSafe(int index, DecimalHolder holder) {
     handleSafe(index);
@@ -497,8 +478,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Store the given value at a particular position in the vector. isSet indicates
-   * whether the value is NULL or not.
+   * Store the given value at a particular position in the vector. isSet indicates whether the value
+   * is NULL or not.
    *
    * @param index position of the new value
    * @param isSet 0 for NULL value, 1 otherwise
@@ -514,9 +495,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, int, long, ArrowBuf)} except that it handles
-   * the case when the position of new value is beyond the current value
-   * capacity of the vector.
+   * Same as {@link #set(int, int, long, ArrowBuf)} except that it handles the case when the
+   * position of new value is beyond the current value capacity of the vector.
    *
    * @param index position of the new value
    * @param isSet 0 for NULL value, 1 otherwise
@@ -533,23 +513,27 @@ public final class DecimalVector extends BaseFixedWidthVector {
     for (int i = 0; i < getValueCount(); ++i) {
       BigDecimal value = getObject(i);
       if (value != null) {
-        ValidateUtil.validateOrThrow(DecimalUtility.checkPrecisionAndScaleNoThrow(value, getPrecision(), getScale()),
-            "Invalid value for DecimalVector at position " + i + ". Value does not fit in precision " +
-                getPrecision() + " and scale " + getScale() + ".");
+        ValidateUtil.validateOrThrow(
+            DecimalUtility.checkPrecisionAndScaleNoThrow(value, getPrecision(), getScale()),
+            "Invalid value for DecimalVector at position "
+                + i
+                + ". Value does not fit in precision "
+                + getPrecision()
+                + " and scale "
+                + getScale()
+                + ".");
       }
     }
   }
 
   /*----------------------------------------------------------------*
-   |                                                                |
-   |                      vector transfer                           |
-   |                                                                |
-   *----------------------------------------------------------------*/
-
+  |                                                                |
+  |                      vector transfer                           |
+  |                                                                |
+  *----------------------------------------------------------------*/
 
   /**
-   * Construct a TransferPair comprising this and a target vector of
-   * the same type.
+   * Construct a TransferPair comprising this and a target vector of the same type.
    *
    * @param ref name of the target vector
    * @param allocator allocator for the target vector
@@ -561,8 +545,7 @@ public final class DecimalVector extends BaseFixedWidthVector {
   }
 
   /**
-   * Construct a TransferPair comprising this and a target vector of
-   * the same type.
+   * Construct a TransferPair comprising this and a target vector of the same type.
    *
    * @param field Field object used by the target vector
    * @param allocator allocator for the target vector
@@ -588,8 +571,8 @@ public final class DecimalVector extends BaseFixedWidthVector {
     DecimalVector to;
 
     public TransferImpl(String ref, BufferAllocator allocator) {
-      to = new DecimalVector(ref, allocator, DecimalVector.this.precision,
-              DecimalVector.this.scale);
+      to =
+          new DecimalVector(ref, allocator, DecimalVector.this.precision, DecimalVector.this.scale);
     }
 
     public TransferImpl(Field field, BufferAllocator allocator) {
